/* ----------------------------------------------------------------------------
 * Copyright (c) Huawei Technologies Co., Ltd. 2013-2019. All rights reserved.
 * Description: BitMap HeadFile
 * Author: Huawei LiteOS Team
 * Create: 2013-01-01
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice, this list of
 * conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list
 * of conditions and the following disclaimer in the documentation and/or other materials
 * provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific prior written
 * permission.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --------------------------------------------------------------------------- */

/**
 * @defgroup los_bitmap Bitmap
 * @ingroup kernel
 */

#ifndef _LOS_BITMAP_H//这段代码是一个条件编译指令块，用于判断是否定义了宏 _LOS_BITMAP_H。如果未定义该宏，则会执行 #ifndef 和 #endif 之间的代码。
#define _LOS_BITMAP_H

#include "los_typedef.h"

#ifdef __cplusplus
#if __cplusplus
extern "C" {
#endif /* __cplusplus */
#endif /* __cplusplus */

/**
 * @ingroup los_bitmap
 * Flag that indicates the invalid bit index.
 *
 * The effective bit index is from 0 to 31.
 */
//由于这里宏定义的值为 32，这个宏的含义是使用 0 到 31 的整数表示位索引值，而 32 则表示无效的位索引值。这种处理方式在计算机编程中比较常见，一般用于标识某些特殊的状态或标志。
#define LOS_INVALID_BIT_INDEX 32

/**
 * @ingroup los_bitmap
 * @brief Set one bit.
 *
 * @par Description:
 * This API is used to set one bit of variable according to the parameter.
 * @attention
 * <ul>
 * <li>When the value of pos is greater than 31, the bit (pos & 0x1f) of bitmap will be set.</li>
 * </ul>
 * @param bitmap   [IN] The bitmap variable pointer.
 * @param pos      [IN] The number bit to be set.
 *
 * @retval None
 * @par Dependency:
 * <ul><li>los_bitmap.h: the header file that contains the API declaration.</li></ul>
 * @see LOS_BitmapClr
 * @since Huawei LiteOS V100R001C00
 */
/*bitmap 是一个指向 UINT32 类型的指针，表示一个位图（或称为位数组）。
pos 是一个 UINT16 类型的参数，表示要设置的位索引。
函数的作用是在给定的位图中将指定的位索引位置设为 1，即将相应位置的比特位设置为 1。*/
VOID LOS_BitmapSet(UINT32 *bitmap, UINT16 pos);

/**
 * @ingroup los_bitmap
 * @brief Clear one bit.
 *
 * @par Description:
 * This API is used to clear one bit of variable according to the parameter.
 * @attention
 * <ul>
 * <li>When the value of pos is greater than 31, the bit (pos & 0x1f) of bitmap will be clear.</li>
 * </ul>
 * @param bitmap   [IN] The bitmap variable pointer.
 * @param pos      [IN] The number bit to be cleared.
 *
 * @retval None.
 * @par Dependency:
 * <ul><li>los_bitmap.h: the header file that contains the API declaration.</li></ul>
 * @see LOS_BitmapSet.
 * @since Huawei LiteOS V100R001C00
 */
/*bitmap 是一个指向 UINT32 类型的指针，表示一个位图（或称为位数组）。
pos 是一个 UINT16 类型的参数，表示要清除的位索引。
函数的作用是在给定的位图中将指定的位索引位置清零，即将相应位置的比特位设置为 0。*/
VOID LOS_BitmapClr(UINT32 *bitmap, UINT16 pos);

/**
 * @ingroup los_bitmap
 * @brief Find the lowest one bit that is set.
 *
 * @par Description:
 * This API is used to find the lowest one bit that is set and return the bit index.
 * @attention
 * None.
 * @param bitmap   [IN] The bitmap variable.
 *
 * @retval #UINT16 The bit index of the lowest one bit that is set.
 * @par Dependency:
 * <ul><li>los_bitmap.h: the header file that contains the API declaration.</li></ul>
 * @see LOS_HighBitGet
 * @since Huawei LiteOS V100R001C00
 */
/*函数的作用是在给定的位图中找到最低位（也就是数值最小的位）的索引，
并返回该索引。如果位图中不存在任何置位的比特位，
则返回 32，即宏 LOS_INVALID_BIT_INDEX 的值。
具体实现可能会涉及位运算和掩码操作，
以找到最低位的索引号。由于函数返回类型为 UINT16，
即 unsigned short 类型，因此返回值为 0 到 31 的整数（包括 0 和 31）或 32*/
UINT16 LOS_LowBitGet(UINT32 bitmap);

/**
 * @ingroup los_bitmap
 * @brief Find the highest one bit that is set.
 *
 * @par Description:
 * This API is used to find the highest one bit that is set and return the bit index.
 * @attention
 * None.
 * @param bitmap   [IN] The bitmap variable.
 *
 * @retval #UINT16 The bit index of the highest one bit that is set.
 * @par Dependency:
 * <ul><li>los_bitmap.h: the header file that contains the API declaration.</li></ul>
 * @see LOS_LowBitGet
 * @since Huawei LiteOS V100R001C00
 */
/*函数的作用是在给定的位图中找到最高位（也就是数值最大的位）的索引，
并返回该索引。如果位图中不存在任何置位的比特位，
则返回 32，即宏 LOS_INVALID_BIT_INDEX 的值。
具体实现可能会涉及位运算和掩码操作，以找到最高位的索引号。
由于函数返回类型为 UINT16，即 unsigned short 类型，
因此返回值为 0 到 31 的整数（包括 0 和 31）或 32。*/
UINT16 LOS_HighBitGet(UINT32 bitmap);

#ifdef __cplusplus
#if __cplusplus
}
#endif /* __cplusplus */
#endif /* __cplusplus */

#endif /* _LOS_BITMAP_H */
